@@ -20,6 +20,8 @@
 #include <QLabel>
 #include <Browser/BreadcrumbList.h>
 
+namespace MainWindow
+{
 class BreadcrumbViewer :public QLabel
 {
     Q_OBJECT
@@ -44,8 +46,9 @@ private:
     void updateText();
 
 private:
-    Browser::BreadcrumbList _activeCrumbs;
+    Browser::BreadcrumbList m_activeCrumbs;
 };
+}
 
 #endif /* BREADCRUMBVIEWER_H */
 
